@@ -26,7 +26,7 @@ INPUT_DIR = "/data1/input"  # data are downloaded to this directory
 INPUT_MOUNT_DIR_PREFIX = "/data1/input-mounted-"  # data are mounted to this directory + bucket name
 
 
-def decode_run_json(input_json_file):
+def decode_run_json(input_json_file, kms_key_id=None):
     """reads a run json file and creates three text files:
     download command list file (commands to download input files from s3)
     input yml file (for cwl/wdl/snakemake run)
@@ -42,7 +42,7 @@ def decode_run_json(input_json_file):
     create_download_command_list(downloadlist_filename, runjson_input)
 
     # create a bucket-mounting command list file
-    create_mount_command_list(mountlist_filename, runjson_input)
+    create_mount_command_list(mountlist_filename, runjson_input, kms_key_id=kms_key_id)
 
     # create an input yml file to be used on awsem
     if language in ['wdl', 'wdl_v1', 'wdl_draft2']:  # wdl
@@ -56,7 +56,13 @@ def decode_run_json(input_json_file):
     create_env_def_file(env_filename, runjson, language)
 
 
-def create_mount_command_list(mountlist_filename, runjson_input):
+def create_mount_command_list(mountlist_filename, runjson_input,
+                              kms_key_id=None):
+    """ This function creates a mount point directory and starts goofys
+        with some default arguments.
+        Note that KMS key arguments are needed for the mount if encryption
+        is enabled.
+    """
     buckets_to_be_mounted = set()
     for category in ["Input_files_data", "Secondary_files_data"]:
         for inkey, v in getattr(runjson_input, category).items():
@@ -65,7 +71,10 @@ def create_mount_command_list(mountlist_filename, runjson_input):
     with open(mountlist_filename, 'w') as f:
         for b in sorted(buckets_to_be_mounted):
             f.write("mkdir -p %s\n" % (INPUT_MOUNT_DIR_PREFIX + b))
-            f.write("goofys -f %s %s &\n" % (b, INPUT_MOUNT_DIR_PREFIX + b))
+            if kms_key_id:
+                f.write("goofys --sse-kms %s -f %s %s &\n" % (kms_key_id, b, INPUT_MOUNT_DIR_PREFIX + b))
+            else:
+                f.write("goofys -f %s %s &\n" % (b, INPUT_MOUNT_DIR_PREFIX + b))
 
 
 def create_download_command_list(downloadlist_filename, runjson_input):
@@ -123,7 +132,7 @@ def determine_key_type(bucket, key, profile):
         # The file itself may be a prefix of another file (e.v. abc.vcf.gz vs abc.vcf.gz.tbi)
         # but it doesn't matter.
         else:
-            return 'File' 
+            return 'File'
     else:
         # data_file is a folder
         return 'Folder'
@@ -185,6 +194,8 @@ def create_env_def_file(env_filename, runjson, language):
         if language in ['wdl', 'wdl_v1', 'wdl_draft2']:
             f_env.write("export WDL_URL={}\n".format(app.wdl_url))
             f_env.write("export MAIN_WDL={}\n".format(app.main_wdl))
+            f_env.write("export WORKFLOW_ENGINE={}\n".format(app.workflow_engine))
+            f_env.write("export RUN_ARGS={}\n".format(app.run_args))
             f_env.write("export WDL_FILES=\"{}\"\n".format(' '.join(app.other_wdl_files.split(','))))
         elif language == 'snakemake':
             f_env.write("export SNAKEMAKE_URL={}\n".format(app.snakemake_url))
@@ -199,6 +210,7 @@ def create_env_def_file(env_filename, runjson, language):
             f_env.write("export CWL_URL={}\n".format(app.cwl_url))
             f_env.write("export MAIN_CWL={}\n".format(app.main_cwl))
             f_env.write("export CWL_FILES=\"{}\"\n".format(' '.join(app.other_cwl_files.split(','))))
+            f_env.write("export RUN_ARGS={}\n".format(app.run_args))
         # other env variables
         env_preserv_str = ''
         docker_env_str = ''
@@ -217,7 +229,7 @@ def download_workflow():
         return
     local_wfdir = os.environ.get('LOCAL_WFDIR')
     subprocess.call(['mkdir', '-p', local_wfdir])
-    
+
     if language in ['wdl', 'wdl_v1', 'wdl_draft2']:
         main_wf = os.environ.get('MAIN_WDL', '')
         wf_files = os.environ.get('WDL_FILES', '')
@@ -239,15 +251,17 @@ def download_workflow():
         wf_files = [wf_files]
     wf_files.append(main_wf)
     wf_url = wf_url.rstrip('/')
-    
+
     print("main workflow file: %s" % main_wf)
     print("workflow files: " + str(wf_files))
-    
+
     s3 = boto3.client('s3')
     for wf_file in wf_files:
         target = "%s/%s" % (local_wfdir, wf_file)
         source = "%s/%s" % (wf_url, wf_file)
+
         if wf_url.startswith('http'):
+            print("downloading via wget (public file)")
             subprocess.call(["wget", "-O" + target, source])
         elif wf_url.startswith('s3'):
             wf_loc = wf_url.replace('s3://', '')
@@ -262,7 +276,7 @@ def download_workflow():
                 targetdir = re.sub('[^/]+$', '', target)
                 subprocess.call(["mkdir", "-p", targetdir])
             s3.download_file(Bucket=bucket_name, Key=key, Filename=target)
-    
+
 
 def read_md5file(md5file):
     with open(md5file, 'r') as md5_f:
@@ -340,7 +354,7 @@ def update_postrun_json_init(json_old, json_new):
 
 
 def update_postrun_json_upload_output(json_old, execution_metadata_file, md5file, json_new,
-                                      language='cwl_v1', strict=True, upload=True):
+                                      language='cwl_v1', strict=True, upload=True, endpoint_url=None):
     """Update postrun json with output files.
     if strict is set false, it does not check execution metadata is required for cwl/wdl."""
     # read old json file and prepare postrunjson skeleton
@@ -348,7 +362,6 @@ def update_postrun_json_upload_output(json_old, execution_metadata_file, md5file
 
     # read md5 file
     md5dict = read_md5file(md5file)
-    print(md5dict)
 
     # read execution metadata file
     if execution_metadata_file:
@@ -357,25 +370,25 @@ def update_postrun_json_upload_output(json_old, execution_metadata_file, md5file
     else:
         execution_metadata = None
     output_files = create_output_files_dict(language, execution_metadata, md5dict, strict=strict)
-    print(output_files)
 
     # create output files for postrun json
     prj.Job.Output.add_output_files(output_files)
 
     # upload output to S3 (this also updates postrun json)
     if upload:
-        upload_output(prj)
+        upload_output(prj, endpoint_url=endpoint_url)
 
     # write to new json file
     write_postrun_json(json_new, prj)
 
 
-def upload_output(prj):
+def upload_output(prj, endpoint_url=None):
     # parsing output_target and uploading output files to output target
-    upload_to_output_target(prj.Job.Output, prj.config.encrypt_s3_upload)
+    upload_to_output_target(prj.Job.Output, prj.config.encrypt_s3_upload,
+                            kms_key_id=prj.config.kms_key_id, endpoint_url=endpoint_url)
 
 
-def upload_to_output_target(prj_out, encrypt_s3_upload=False):
+def upload_to_output_target(prj_out, encrypt_s3_upload=False, kms_key_id=None, endpoint_url=None):
     # parsing output_target and uploading output files to output target
     output_bucket = prj_out.output_bucket_directory
     output_argnames = prj_out.output_files.keys()
@@ -390,7 +403,7 @@ def upload_to_output_target(prj_out, encrypt_s3_upload=False):
             target.parse_custom_target(k, output_target[k])
             if target.is_valid:
                 print("Target is valid. Uploading..")
-                target.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload)
+                target.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload, endpoint_url=endpoint_url)
             else:
                 raise Exception("Invalid target %s -> %s: failed to upload" % k, output_target[k])
         else:
@@ -399,9 +412,10 @@ def upload_to_output_target(prj_out, encrypt_s3_upload=False):
             target.parse_cwl_target(k, output_target.get(k, ''), prj_out.output_files)
             if target.is_valid:
                 print("Target is valid. Uploading..")
-                target.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload)
+                target.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload,
+                                    kms_key_id=kms_key_id, endpoint_url=endpoint_url)
                 prj_out.output_files[k].add_target(target.dest)
-    
+
                 # upload secondary files
                 secondary_output_files = prj_out.output_files[k].secondaryFiles
                 if secondary_output_files:
@@ -409,7 +423,8 @@ def upload_to_output_target(prj_out, encrypt_s3_upload=False):
                     stlist.parse_target_values(prj_out.secondary_output_target.get(k, []))
                     stlist.reorder_by_source([sf.path for sf in secondary_output_files])
                     for st in stlist.secondary_targets:
-                        st.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload)
+                        st.upload_to_s3(encrypt_s3_upload=encrypt_s3_upload,
+                                        kms_key_id=kms_key_id, endpoint_url=endpoint_url)
                     for i, sf in enumerate(secondary_output_files):
                         sf.add_target(stlist.secondary_targets[i].dest)
             else:
@@ -426,9 +441,9 @@ def update_postrun_json_final(json_old, json_new, logfile=None):
     """Update postrun json with status, time stamps, parsed commands,
     input/tmp/output sizes"""
     prj = read_postrun_json(json_old)
-    
+
     postrun_json_final(prj, logfile=logfile)
-    
+
     # write to new json file
     write_postrun_json(json_new, prj)
 
@@ -470,4 +485,6 @@ def upload_postrun_json(jsonfile):
     }
     if prj.config.encrypt_s3_upload:
         upload_arg.update({"ServerSideEncryption": "aws:kms"})
+        if prj.config.kms_key_id:
+            upload_arg['SSEKMSKeyId'] = prj.config.kms_key_id
     s3.put_object(**upload_arg)
