@@ -55,9 +55,9 @@ class AwsemRunJsonJob(SerializableObject):
             setattr(self, k, v)
 
     @property
-    def start_time_as_str(self):
+    def start_time_as_datetime(self):
         if not self.start_time:
-            return ''
+            return None
         else:
             return datetime.strptime(self.start_time, AWSEM_TIME_STAMP_FORMAT)
 
@@ -70,7 +70,7 @@ class AwsemRunJsonLog(SerializableObject):
 class AwsemRunJsonApp(SerializableObject):
     def __init__(self, App_name=None, App_version=None, language=None,
                  cwl_url=None, main_cwl=None, other_cwl_files=None,
-                 wdl_url=None, main_wdl=None, other_wdl_files=None,
+                 wdl_url=None, main_wdl=None, other_wdl_files=None, workflow_engine=None, run_args=None,
                  container_image=None, command=None,
                  snakemake_url=None, main_snakemake=None, other_snakemake_files=None):
         self.App_name = App_name
@@ -82,6 +82,8 @@ class AwsemRunJsonApp(SerializableObject):
         self.wdl_url = wdl_url
         self.main_wdl = main_wdl
         self.other_wdl_files = other_wdl_files
+        self.workflow_engine = workflow_engine
+        self.run_args = run_args
         self.container_image = container_image
         self.command = command
         self.snakemake_url = snakemake_url
@@ -289,7 +291,7 @@ class AwsemPostRunJsonJob(AwsemRunJsonJob):
                  start_time=None, end_time=None, status=None, Log=None,
                  total_input_size=None, total_output_size=None, total_tmp_size=None,
                  # older postrunjsons don't have these fields
-                 filesystem='', instance_id='',
+                 filesystem='', instance_id='', instance_availablity_zone='',
                  Metrics=None, strict=True):
         if strict:
             if App is None or Input is None or Output is None or not JOBID or start_time is None:
@@ -300,6 +302,7 @@ class AwsemPostRunJsonJob(AwsemRunJsonJob):
         self.status = status
         self.filesystem = filesystem
         self.instance_id = instance_id
+        self.instance_availablity_zone = instance_availablity_zone
         self.total_input_size = total_input_size
         self.total_output_size = total_output_size
         self.total_tmp_size = total_tmp_size
@@ -309,7 +312,7 @@ class AwsemPostRunJsonJob(AwsemRunJsonJob):
         self.Output = AwsemPostRunJsonOutput(**Output)
 
     @property
-    def end_time_as_str(self):
+    def end_time_as_datetime(self):
         try:
             return datetime.strptime(self.end_time, AWSEM_TIME_STAMP_FORMAT)
         except:
