@@ -7,6 +7,7 @@ import boto3
 import botocore
 import copy
 import re
+from random import choice
 from . import create_logger
 from datetime import datetime, timedelta
 from .utils import (
@@ -23,7 +24,7 @@ from .vars import (
     DEFAULT_ROOT_EBS_SIZE,
     TIBANNA_AWSF_DIR,
     DEFAULT_AWSF_IMAGE,
-    AWS_REGION_NAMES
+    S3_ENCRYT_KEY_ID,
 )
 from .job import Jobs
 from .exceptions import (
@@ -171,6 +172,8 @@ class Args(SerializableObject):
                 self.wdl_directory_url = ''
             if not hasattr(self, 'workflow_engine'):
                 self.workflow_engine = 'cromwell'
+            if not hasattr(self, 'run_args'):
+                self.run_args = ''
             if not self.wdl_directory_local and not self.wdl_directory_url:
                 errmsg = "either %s or %s must be provided in args" % ('wdl_directory_url', 'wdl_directory_local')
                 raise MissingFieldInInputJsonException(errmsg)
@@ -203,6 +206,8 @@ class Args(SerializableObject):
                 self.cwl_directory_local = ''
             if not hasattr(self, 'cwl_directory_url'):
                 self.cwl_directory_url = ''
+            if not hasattr(self, 'run_args'):
+                self.run_args = ''
             if not self.cwl_directory_local and not self.cwl_directory_url:
                 errmsg = "either %s or %s must be provided in args" % ('cwl_directory_url', 'cwl_directory_local')
                 raise MissingFieldInInputJsonException(errmsg)
@@ -276,7 +281,7 @@ class Config(SerializableObject):
             if not hasattr(self, 'instance_type') and (not hasattr(self, 'cpu') or not hasattr(self, 'mem')):
                 self.use_benchmark = True
             elif not hasattr(self, 'ebs_size'):
-                self.use_benchmark = True 
+                self.use_benchmark = True
             else:
                 self.use_benchmark = False
         # fill in default
@@ -301,8 +306,16 @@ class Config(SerializableObject):
         if not hasattr(self, 'public_postrun_json'):
             self.public_postrun_json = False
             # 4dn will use 'true' --> this will automatically be added by start_run_awsem
+
+        # TODO: default to env variable here?
         if not hasattr(self, 'encrypt_s3_upload'):
             self.encrypt_s3_upload = False
+        if not hasattr(self, 'kms_key_id'):
+            if S3_ENCRYT_KEY_ID:
+                self.kms_key_id = S3_ENCRYT_KEY_ID
+                self.encrypt_s3_upload = True
+            else:
+                self.kms_key_id = None
         if not hasattr(self, 'root_ebs_size'):
             self.root_ebs_size = DEFAULT_ROOT_EBS_SIZE
         if not hasattr(self, 'awsf_image'):
@@ -311,9 +324,16 @@ class Config(SerializableObject):
             self.mem_as_is = False
         if not hasattr(self, 'ebs_size_as_is'):  # if false, add 5GB overhead
             self.ebs_size_as_is = False
-        if not hasattr(self, 'ami_id'): 
+        if not hasattr(self, 'ami_id'):
             self.ami_id = AMI_ID
-        
+        # special handling for subnet, SG if not set already pull from env
+        # values from config take priority - omit them to get these values from lambda
+        if not self.subnet and os.environ.get('SUBNETS', ''):
+            possible_subnets = os.environ['SUBNETS'].split(',')
+            self.subnet = possible_subnets  # propagate all subnets here, pick one in launch_args
+        if not self.security_group and os.environ.get('SECURITY_GROUPS', ''):
+            self.security_group = os.environ['SECURITY_GROUPS'].split(',')[0]
+
     def fill_internal(self):
         # fill internally-used fields (users cannot specify these fields)
         # script url
@@ -380,12 +400,22 @@ class Execution(object):
         instance_type_dlist = []
         # user directly specified instance type
         if instance_type:
-            if self.user_specified_EBS_optimized:
-                instance_type_dlist.append({'instance_type': instance_type,
-                                            'EBS_optimized': self.user_specified_EBS_optimized})
-            else:
-                instance_type_dlist.append({'instance_type': instance_type,
-                                            'EBS_optimized': False})
+            if isinstance(instance_type, str):
+                if self.user_specified_EBS_optimized:
+                    instance_type_dlist.append({'instance_type': instance_type,
+                                                'EBS_optimized': self.user_specified_EBS_optimized})
+                else:
+                    instance_type_dlist.append({'instance_type': instance_type,
+                                                'EBS_optimized': False})
+            elif isinstance(instance_type, list):
+                for potential_type in instance_type:
+                    if self.user_specified_EBS_optimized:
+                        instance_type_dlist.append({'instance_type': potential_type,
+                                                    'EBS_optimized': self.user_specified_EBS_optimized})
+                    else:
+                        instance_type_dlist.append({'instance_type': potential_type,
+                                                    'EBS_optimized': False})
+
         # user specified mem and cpu
         if self.cfg.mem and self.cfg.cpu:
             if self.cfg.mem_as_is:
@@ -403,7 +433,7 @@ class Execution(object):
             instance_type_dlist.append(self.benchmark)
         self.instance_type_list = [i['instance_type'] for i in instance_type_dlist]
         self.instance_type_info = {i['instance_type']: i for i in instance_type_dlist}
-        self.current_instance_type_index = 0  # choose the first one initially
+        self.choose_next_instance_type()  # randomly choose
 
     @property
     def current_instance_type(self):
@@ -420,9 +450,10 @@ class Execution(object):
             return ''
 
     def choose_next_instance_type(self):
-        self.current_instance_type_index += 1
-        if len(self.instance_type_list) <= self.current_instance_type_index:
-            raise Exception("No more instance type available that matches the criteria")
+        """ Previously we would start to launch jobs at index 0 and increment, but now we will randomly pick
+            from the list
+        """
+        self.current_instance_type_index = choice(range(0, len(self.instance_type_list)))
 
     def update_config_instance_type(self):
         # deal with missing fields
@@ -462,7 +493,6 @@ class Execution(object):
         if self.cfg.ebs_size < 10:
             self.cfg.ebs_size = 10
 
-
     def get_input_size_in_bytes(self):
         input_size_in_bytes = dict()
         input_plus_secondary_files = copy.deepcopy(self.args.input_files)
@@ -527,27 +557,30 @@ class Execution(object):
             try:
                 return func(*args, **kwargs)
             except Exception as e:
-                if 'InsufficientInstanceCapacity' in str(e) or 'InstanceLimitExceeded' in str(e):
+                error_response = str(e)
+                # No instance capacity, too many instances or incompatible subnet/instance configuration - try again
+                if ('InsufficientInstanceCapacity' in error_response or 'InstanceLimitExceeded' in error_response or
+                        'is not supported in your requested Availability Zone' in error_response):
                     behavior = self.cfg.behavior_on_capacity_limit
                     if behavior == 'fail':
                         errmsg = "Instance limit exception - use 'behavior_on_capacity_limit' option" + \
                                  "to change the behavior to wait_and_retry, other_instance_types," + \
-                                 "or retry_without_spot. %s" % str(e)
+                                 "or retry_without_spot. %s" % error_response
                         raise EC2InstanceLimitException(errmsg)
                     elif behavior == 'wait_and_retry':
-                        errmsg = "Instance limit exception - wait and retry later: %s" % str(e)
+                        errmsg = "Instance limit exception - wait and retry later: %s" % error_response
                         raise EC2InstanceLimitWaitException(errmsg)
                     elif behavior == 'other_instance_types':
                         try:
                             self.choose_next_instance_type()
-                        except Exception as e2:
+                        except Exception as e2:  # should never hit now as we will continue indefinitely
                             raise EC2InstanceLimitException(str(e2))
                         self.update_config_instance_type()
                         return 'continue'
                     elif behavior == 'retry_without_spot':
                         if not self.cfg.spot_instance:
                             errmsg = "'behavior_on_capacity_limit': 'retry_without_spot' works only with " + \
-                                     "'spot_instance' : true. %s" % str(e)
+                                     "'spot_instance' : true. %s" % error_response
                             raise Exception(errmsg)
                         else:
                             self.cfg.spot_instance = False
@@ -581,7 +614,8 @@ class Execution(object):
                 'main_wdl': args.wdl_main_filename,
                 'other_wdl_files': ','.join(args.wdl_child_filenames),
                 'wdl_url': args.wdl_directory_url,
-                'workflow_engine': args.workflow_engine
+                'workflow_engine': args.workflow_engine,
+                'run_args': args.run_args
             })
         elif args.language == 'snakemake':
             app.update({
@@ -601,6 +635,7 @@ class Execution(object):
                 'main_cwl': args.cwl_main_filename,
                 'other_cwl_files': ','.join(args.cwl_child_filenames),
                 'cwl_url': args.cwl_directory_url,
+                'run_args': args.run_args
             })
         pre.update({'Job': {'JOBID': jobid,
                             'App': app,
@@ -664,6 +699,8 @@ class Execution(object):
         }
         if self.cfg.encrypt_s3_upload:
             upload_args.update({'ServerSideEncryption': 'aws:kms'})
+            if self.cfg.kms_key_id:
+                upload_args['SSEKMSKeyId'] = self.cfg.kms_key_id
         try:
             s3 = boto3.client('s3')
         except Exception as e:
@@ -691,6 +728,8 @@ class Execution(object):
         str += " -l $LOGBUCKET"
         str += " -V {version}".format(version=__version__)
         str += " -A {awsf_image}".format(awsf_image=cfg.awsf_image)
+        if cfg.kms_key_id:
+            str += " -k %s" % cfg.kms_key_id
         if cfg.password:
             str += " -p {}".format(cfg.password)
         if profile:
@@ -751,7 +790,10 @@ class Execution(object):
         if self.cfg.security_group:
             largs.update({'SecurityGroupIds': [self.cfg.security_group]})
         if self.cfg.subnet:
-            largs.update({'SubnetId': self.cfg.subnet})
+            if isinstance(self.cfg.subnet, str):
+                largs.update({'SubnetId': self.cfg.subnet})
+            elif isinstance(self.cfg.subnet, list):
+                largs.update({'SubnetId': choice(self.cfg.subnet)})  # if given multiple, pick one randomly
         if self.dryrun:
             largs.update({'DryRun': True})
         return largs
@@ -776,8 +818,8 @@ class Execution(object):
                 break
             except:
                 continue
-        return({'instance_id': self.instance_id, 
-                'instance_ip': instance_ip, 
+        return({'instance_id': self.instance_id,
+                'instance_ip': instance_ip,
                 'availability_zone' : availability_zone,
                 'start_time': self.get_start_time()})
 
@@ -935,8 +977,8 @@ class Execution(object):
             DashboardName=dashboard_name,
             DashboardBody=json.dumps(body)
         )
-       
-    
+
+
 def upload_workflow_to_s3(unicorn_input):
     """input is a UnicornInput object"""
     args = unicorn_input.args
